@@ -23,6 +23,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "Command.h"
 #include "Conversation.h"
 #include "text/DisplayText.h"
+#include "Endpoint.h"
 #include "shader/FillShader.h"
 #include "text/Font.h"
 #include "text/FontSet.h"
@@ -89,7 +90,7 @@ ConversationPanel::ConversationPanel(PlayerInfo &player, const Conversation &con
 		subs[it.first].swap(it.second);
 	if(ship)
 	{
-		subs["<ship>"] = ship->Name();
+		subs["<ship>"] = ship->GivenName();
 		subs["<model>"] = ship->DisplayModelName();
 	}
 
@@ -229,10 +230,11 @@ void ConversationPanel::Draw()
 			if(index == choice)
 				FillShader::Fill(center + Point(-5, 0), size + Point(30, 0), selectionColor);
 			AddZone(zone, [this, index](){ this->ClickChoice(index); });
-			++index;
 
 			font.Draw(label, point + Point(-15, 0), dim);
-			point = paragraph.Draw(point, bright);
+			point = paragraph.Draw(point, conversation.ChoiceIsActive(node, MapChoice(index)) ? bright : dim);
+
+			++index;
 		}
 	}
 	// Store the total height of the text.
@@ -254,7 +256,7 @@ bool ConversationPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &comm
 	if(command.Has(Command::MAP) && (!choices.empty() || node < 0))
 	{
 		sound = UI::UISound::NONE;
-		GetUI()->Push(new MapDetailPanel(player, system, true));
+		GetUI().Push(new MapDetailPanel(player, system, true));
 	}
 	if(node < 0)
 	{
@@ -329,11 +331,26 @@ bool ConversationPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &comm
 	else if(key == SDLK_DOWN && choice + 1 < static_cast<int>(choices.size()))
 		++choice;
 	else if((key == SDLK_RETURN || key == SDLK_KP_ENTER) && isNewPress && choice < static_cast<int>(choices.size()))
-		Goto(conversation.NextNodeForChoice(node, MapChoice(choice)), choice);
+	{
+		if(conversation.ChoiceIsActive(node, MapChoice(choice)))
+			Goto(conversation.NextNodeForChoice(node, MapChoice(choice)), choice);
+		else
+			sound = UI::UISound::FAILURE;
+	}
 	else if(key >= '1' && key < static_cast<SDL_Keycode>('1' + choices.size()))
-		Goto(conversation.NextNodeForChoice(node, MapChoice(key - '1')), key - '1');
+	{
+		if(conversation.ChoiceIsActive(node, MapChoice(key - '1')))
+			Goto(conversation.NextNodeForChoice(node, MapChoice(key - '1')), key - '1');
+		else
+			sound = UI::UISound::FAILURE;
+	}
 	else if(key >= SDLK_KP_1 && key < static_cast<SDL_Keycode>(SDLK_KP_1 + choices.size()))
-		Goto(conversation.NextNodeForChoice(node, MapChoice(key - SDLK_KP_1)), key - SDLK_KP_1);
+	{
+		if(conversation.ChoiceIsActive(node, MapChoice(key - SDLK_KP_1)))
+			Goto(conversation.NextNodeForChoice(node, MapChoice(key - SDLK_KP_1)), key - SDLK_KP_1);
+		else
+			sound = UI::UISound::FAILURE;
+	}
 	else
 		return false;
 
@@ -375,7 +392,7 @@ bool ConversationPanel::Hover(int x, int y)
 void ConversationPanel::Goto(int index, int selectedChoice)
 {
 	const ConditionsStore &conditions = player.Conditions();
-	Format::ConditionGetter getter = [&conditions](const std::string &str, size_t start, size_t length) -> int64_t
+	Format::ConditionGetter getter = [&conditions](const string &str, size_t start, size_t length) -> int64_t
 	{
 		return conditions.Get(str.substr(start, length));
 	};
@@ -450,7 +467,7 @@ void ConversationPanel::Goto(int index, int selectedChoice)
 	// This is a safeguard in case of logic errors, to ensure we don't set the player name.
 	if(choices.empty() && conversation.Choices(node) != 0)
 	{
-		node = Conversation::DECLINE;
+		node = Endpoint::DECLINE;
 	}
 	this->choice = 0;
 }
@@ -464,24 +481,24 @@ void ConversationPanel::Exit()
 	if(useTransactions)
 		player.FinishTransaction();
 
-	GetUI()->Pop(this);
+	GetUI().Pop(this);
 	// Some conversations may be offered from an NPC, e.g. an assisting or
 	// boarding mission's `on offer`, or from completing a mission's NPC
 	// block (e.g. scanning or boarding or killing all required targets).
-	if(node == Conversation::DIE || node == Conversation::EXPLODE)
+	if(node == Endpoint::DIE || node == Endpoint::EXPLODE)
 		player.Die(node, ship);
 	else if(ship)
 	{
 		// A forced-launch ending (LAUNCH, FLEE, or DEPART) destroys any NPC.
-		if(Conversation::RequiresLaunch(node))
+		if(Endpoint::RequiresLaunch(node))
 			ship->Destroy();
 		// Only show the BoardingPanel for a hostile NPC that is being boarded.
 		// (NPC completion conversations can result from non-boarding events.)
 		// TODO: Is there a better / more robust boarding check than relative position?
-		else if((node != Conversation::ACCEPT || player.CaptureOverriden(ship)) && ship->GetGovernment()->IsEnemy()
+		else if((node != Endpoint::ACCEPT || player.CaptureOverriden(ship)) && ship->GetGovernment()->IsEnemy()
 				&& !ship->IsDestroyed() && ship->IsDisabled()
 				&& ship->Position().Distance(player.Flagship()->Position()) <= 1.)
-			GetUI()->Push(new BoardingPanel(player, ship));
+			GetUI().Push(new BoardingPanel(player, ship));
 	}
 	// Call the exit response handler to manage the conversation's effect
 	// on the player's missions, or force takeoff from a planet.
@@ -502,7 +519,10 @@ void ConversationPanel::ClickName(int side)
 // The player just clicked on a conversation choice.
 void ConversationPanel::ClickChoice(int index)
 {
-	Goto(conversation.NextNodeForChoice(node, MapChoice(index)), index);
+	if(conversation.ChoiceIsActive(node, MapChoice(index)))
+		Goto(conversation.NextNodeForChoice(node, MapChoice(index)), index);
+	else
+		UI::PlaySound(UI::UISound::FAILURE);
 }
 
 
