@@ -435,6 +435,8 @@ void GameData::Revert()
 	objects.substitutions.Revert(defaultSubstitutions);
 	objects.wormholes.Revert(defaultWormholes);
 	objects.persons.Revert(defaultPersons);
+	for(auto &it : objects.persons)
+		it.second.Restore();
 
 	politics.Reset();
 	purchases.clear();
@@ -578,7 +580,7 @@ void GameData::AddPurchase(const System &system, const string &commodity, int to
 
 
 // Apply the given change to the universe.
-void GameData::Change(const DataNode &node, const PlayerInfo &player)
+void GameData::Change(const DataNode &node, PlayerInfo &player)
 {
 	objects.Change(node, player);
 }
@@ -594,6 +596,13 @@ void GameData::UpdateSystems()
 
 
 
+void GameData::RecomputeWormholeRequirements()
+{
+	objects.RecomputeWormholeRequirements();
+}
+
+
+
 void GameData::AddJumpRange(double neighborDistance)
 {
 	objects.neighborDistances.insert(neighborDistance);
@@ -697,6 +706,20 @@ const Set<Interface> &GameData::Interfaces()
 
 
 
+const Set<Message::Category> &GameData::MessageCategories()
+{
+	return objects.messageCategories;
+}
+
+
+
+const Set<Message> &GameData::Messages()
+{
+	return objects.messages;
+}
+
+
+
 const Set<Minable> &GameData::Minables()
 {
 	return objects.minables;
@@ -809,6 +832,13 @@ const Set<Wormhole> &GameData::Wormholes()
 
 
 
+const std::set<std::string> &GameData::UniverseWormholeRequirements()
+{
+	return objects.universeWormholeRequirements;
+}
+
+
+
 const Government *GameData::PlayerGovernment()
 {
 	return playerGovernment;
