@@ -15,7 +15,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 
 #include "Messages.h"
 
-#include "Color.h"
+#include "DataWriter.h"
 #include "GameData.h"
 
 #include <mutex>
@@ -27,34 +27,32 @@ namespace {
 
 	mutex incomingMutex;
 
-	vector<pair<string, Messages::Importance>> incoming;
+	vector<pair<string, const Message::Category *>> incoming;
 	vector<Messages::Entry> recent;
-	deque<pair<string, Messages::Importance>> logged;
+	deque<pair<string, const Message::Category *>> logged;
 }
 
 
 
-// Add a message to the list along with its level of importance
-// When forced, the message is forcibly added to the log, but not to the list.
-void Messages::Add(const string &message, Importance importance, bool force)
+// Add a message to the list along with its level of importance.
+void Messages::Add(const Message &message)
 {
-	lock_guard<mutex> lock(incomingMutex);
-	incoming.emplace_back(message, importance);
-	AddLog(message, importance, force);
-}
+	const Message::Category *category = message.GetCategory();
+	if(!category)
+		return;
+	string text = message.Text();
 
-
-
-// Add a message to the log. For messages meant to be shown
-// also on the main panel, use Add instead.
-void Messages::AddLog(const string &message, Importance importance, bool force)
-{
-	if(force || logged.empty() || message != logged.front().first || importance == Importance::HighestDuplicating)
+	if(category->AllowsLogDuplicates() || logged.empty() || text != logged.front().first)
 	{
-		logged.emplace_front(message, importance);
+		logged.emplace_front(text, category);
 		if(logged.size() > MAX_LOG)
 			logged.pop_back();
 	}
+
+	if(category->LogOnly())
+		return;
+	lock_guard<mutex> lock(incomingMutex);
+	incoming.emplace_back(text, category);
 }
 
 
@@ -77,14 +75,11 @@ const vector<Messages::Entry> &Messages::Get(int step, int animationDuration)
 	}
 
 	// Load the incoming messages.
-	for(const pair<string, Importance> &item : incoming)
+	for(const auto &[message, category] : incoming)
 	{
-		const string &message = item.first;
-		Importance importance = item.second;
-
 		// If this message is not important and it is already being shown in the
 		// list, ignore it.
-		if(importance == Importance::Low || importance == Importance::HighestNoRepeat)
+		if(category->MainDuplicatesStrategy() == Message::Category::DuplicatesStrategy::KEEP_OLD)
 		{
 			bool skip = false;
 			for(const Messages::Entry &entry : recent)
@@ -102,11 +97,11 @@ const vector<Messages::Entry> &Messages::Get(int step, int animationDuration)
 				it.step -= 60;
 			// For each incoming message, if it exactly matches an existing message,
 			// replace that one with this new one by scheduling the old one for removal.
-			if(importance != Importance::Low && importance != Importance::HighestNoRepeat
-					&& importance != Importance::HighestDuplicating && it.message == message && it.deathStep < 0)
+			if(category->MainDuplicatesStrategy() == Message::Category::DuplicatesStrategy::KEEP_NEW
+					&& it.message == message && it.deathStep < 0)
 				it.deathStep = step + animationDuration;
 		}
-		recent.emplace_back(step, message, importance);
+		recent.emplace_back(step, message, category);
 	}
 	incoming.clear();
 	return recent;
@@ -114,13 +109,20 @@ const vector<Messages::Entry> &Messages::Get(int step, int animationDuration)
 
 
 
-const deque<pair<string, Messages::Importance>> &Messages::GetLog()
+const deque<pair<string, const Message::Category *>> &Messages::GetLog()
 {
 	return logged;
 }
 
 
 
+void Messages::ClearLog()
+{
+	logged.clear();
+}
+
+
+
 // Reset the messages (i.e. because a new game was loaded).
 void Messages::Reset()
 {
@@ -132,24 +134,28 @@ void Messages::Reset()
 
 
 
-// Get color that should be used for drawing messages of given importance.
-const Color *Messages::GetColor(Importance importance, bool isLogPanel)
+void Messages::LoadLog(const DataNode &node)
 {
-	string prefix = isLogPanel ? "message log importance " : "message importance ";
-	switch(importance)
+	for(const DataNode &child : node)
 	{
-		case Messages::Importance::Highest:
-		case Messages::Importance::HighestNoRepeat:
-		case Messages::Importance::HighestDuplicating:
-			return GameData::Colors().Get(prefix + "highest");
-		case Messages::Importance::High:
-			return GameData::Colors().Get(prefix + "high");
-		case Messages::Importance::Info:
-			return GameData::Colors().Get(prefix + "info");
-		case Messages::Importance::Daily:
-			return GameData::Colors().Get(prefix + "daily");
-		case Messages::Importance::Low:
-		default:
-			return GameData::Colors().Get(prefix + "low");
+		if(child.Size() < 2)
+		{
+			child.PrintTrace("Skipping message log entry without category:");
+			continue;
+		}
+		logged.emplace_front(child.Token(1), GameData::MessageCategories().Get(child.Token(0)));
 	}
 }
+
+
+
+void Messages::SaveLog(DataWriter &out)
+{
+	out.Write("message log");
+	out.BeginChild();
+	{
+		for(auto it = logged.rbegin(); it != logged.rend(); ++it)
+			out.Write(it->second->TrueName(), it->first);
+	}
+	out.EndChild();
+}
