@@ -21,7 +21,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "audio/Audio.h"
 #include "Command.h"
 #include "CoreStartData.h"
-#include "Dialog.h"
+#include "DialogPanel.h"
 #include "text/DisplayText.h"
 #include "shader/FillShader.h"
 #include "text/Font.h"
@@ -138,11 +138,7 @@ MissionPanel::MissionPanel(PlayerInfo &player)
 	while(acceptedIt != accepted.end() && !acceptedIt->IsVisible())
 		++acceptedIt;
 
-	description = make_shared<TextArea>();
-	description->SetFont(FontSet::Get(14));
-	description->SetAlignment(Alignment::JUSTIFIED);
-	description->SetColor(*GameData::Colors().Get("bright"));
-	description->SetRect(missionInterface->GetBox("description"));
+	InitTextArea();
 
 	// Select the first available or accepted mission in the currently selected
 	// system, or along the travel plan.
@@ -182,11 +178,7 @@ MissionPanel::MissionPanel(const MapPanel &panel)
 	while(acceptedIt != accepted.end() && !acceptedIt->IsVisible())
 		++acceptedIt;
 
-	description = make_shared<TextArea>();
-	description->SetFont(FontSet::Get(14));
-	description->SetAlignment(Alignment::JUSTIFIED);
-	description->SetColor(*GameData::Colors().Get("bright"));
-	description->SetRect(missionInterface->GetBox("description"));
+	InitTextArea();
 
 	// Select the first available or accepted mission in the currently selected
 	// system, or along the travel plan.
@@ -212,13 +204,13 @@ void MissionPanel::Step()
 	// If a job or mission that launches the player triggers,
 	// immediately close the map.
 	if(player.ShouldLaunch())
-		GetUI()->Pop(this);
+		GetUI().Pop(this);
 
-	if(GetUI()->IsTop(this) && player.GetPlanet() && player.GetDate() >= player.StartData().GetDate() + 12)
+	if(GetUI().IsTop(this) && player.GetPlanet() && player.GetDate() >= player.StartData().GetDate() + 12)
 		DoHelp("map advanced");
 	DoHelp("jobs");
 
-	if(GetUI()->IsTop(this) && !fromMission)
+	if(GetUI().IsTop(this) && !fromMission)
 	{
 		Mission *mission = player.MissionToOffer(Mission::JOB_BOARD);
 		if(mission)
@@ -232,7 +224,7 @@ void MissionPanel::Step()
 			specialSystem = mission->IsVisible() ? mission->Destination()->GetSystem() : nullptr;
 			CenterOnSystem(specialSystem ? specialSystem : player.GetSystem());
 
-			mission->Do(Mission::OFFER, player, GetUI());
+			mission->Do(Mission::OFFER, player, &GetUI());
 		}
 		else
 		{
@@ -322,6 +314,14 @@ void MissionPanel::Draw()
 
 
 
+void MissionPanel::UpdateTooltipActivation()
+{
+	MapPanel::UpdateTooltipActivation();
+	tooltip.UpdateActivationCount();
+}
+
+
+
 // Only override the ones you need; the default action is to return false.
 bool MissionPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress)
 {
@@ -340,8 +340,8 @@ bool MissionPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 	else if(key == 'A' || (key == 'a' && (mod & KMOD_SHIFT)))
 	{
 		if(acceptedIt != accepted.end() && acceptedIt->IsVisible())
-			GetUI()->Push(new Dialog(this, &MissionPanel::AbortMission,
-				"Abort mission \"" + acceptedIt->Name() + "\"?"));
+			GetUI().Push(new DialogPanel(this, &MissionPanel::AbortMission,
+				"Abort mission \"" + acceptedIt->DisplayName() + "\"?"));
 		return true;
 	}
 	else if(key == SDLK_LEFT && availableIt == available.end())
@@ -663,6 +663,31 @@ bool MissionPanel::Scroll(double dx, double dy)
 
 
 
+void MissionPanel::Resize()
+{
+	ResizeTextArea();
+}
+
+
+
+void MissionPanel::InitTextArea()
+{
+	description = make_shared<TextArea>();
+	description->SetFont(FontSet::Get(14));
+	description->SetAlignment(Alignment::JUSTIFIED);
+	description->SetColor(*GameData::Colors().Get("bright"));
+	ResizeTextArea();
+}
+
+
+
+void MissionPanel::ResizeTextArea() const
+{
+	description->SetRect(missionInterface->GetBox("description"));
+}
+
+
+
 void MissionPanel::SetSelectedScrollAndCenter(bool immediate)
 {
 	// Auto select the destination system for the current mission.
@@ -689,33 +714,12 @@ void MissionPanel::SetSelectedScrollAndCenter(bool immediate)
 
 void MissionPanel::DrawKey() const
 {
-	const Sprite *back = SpriteSet::Get("ui/mission key");
-	SpriteShader::Draw(back, Screen::BottomLeft() + .5 * Point(back->Width(), -back->Height()));
-
-	const Font &font = FontSet::Get(14);
-	Point angle = Point(1., 1.).Unit();
-
-	const int ROWS = 5;
-	Point pos(Screen::Left() + 10., Screen::Bottom() - ROWS * 20. + 5.);
-	Point pointerOff(5., 5.);
-	Point textOff(8., -.5 * font.Height());
-
-	const Set<Color> &colors = GameData::Colors();
-	const Color &bright = *colors.Get("bright");
-	const Color &dim = *colors.Get("dim");
-	const Color COLOR[ROWS] = {
-		*colors.Get("available job"),
-		*colors.Get("unavailable job"),
-		*colors.Get("active mission"),
-		*colors.Get("blocked mission"),
-		*colors.Get("waypoint")
-	};
-	static const string LABEL[ROWS] = {
-		"Available job; can accept",
-		"Too little space to accept",
-		"Active job; go here to complete",
-		"Has unfinished requirements",
-		"System of importance"
+	static const string CONDITIONS[] = {
+		"available",
+		"unavailable",
+		"active",
+		"blocked",
+		"waypoint"
 	};
 	int selected = -1;
 	if(availableIt != available.end())
@@ -723,12 +727,10 @@ void MissionPanel::DrawKey() const
 	if(acceptedIt != accepted.end() && acceptedIt->Destination())
 		selected = 2 + !IsSatisfied(*acceptedIt);
 
-	for(int i = 0; i < ROWS; ++i)
-	{
-		PointerShader::Draw(pos + pointerOff, angle, 10.f, 18.f, 0.f, COLOR[i]);
-		font.Draw(LABEL[i], pos + textOff, i == selected ? bright : dim);
-		pos.Y() += 20.;
-	}
+	Information info;
+	if(selected >= 0)
+		info.SetCondition(CONDITIONS[selected]);
+	GameData::Interfaces().Get("map: mission view: key")->Draw(info);
 }
 
 
@@ -739,6 +741,7 @@ void MissionPanel::DrawMissionSystem(const Mission &mission, const Color &color)
 {
 	auto toVisit = set<const System *>{mission.Waypoints()};
 	toVisit.insert(mission.MarkedSystems().begin(), mission.MarkedSystems().end());
+	toVisit.insert(mission.TrackedSystems().begin(), mission.TrackedSystems().end());
 	for(const Planet *planet : mission.Stopovers())
 		toVisit.insert(planet->GetSystem());
 	auto hasVisited = set<const System *>{mission.VisitedWaypoints()};
@@ -848,7 +851,7 @@ Point MissionPanel::DrawPanel(Point pos, const string &label, int entries, bool
 
 
 
-Point MissionPanel::DrawList(const list<Mission> &list, Point pos, const std::list<Mission>::const_iterator &selectIt,
+Point MissionPanel::DrawList(const list<Mission> &missionList, Point pos, const list<Mission>::const_iterator &selectIt,
 	bool separateDeadlineOrPossible) const
 {
 	const Font &font = FontSet::Get(14);
@@ -859,7 +862,7 @@ Point MissionPanel::DrawList(const list<Mission> &list, Point pos, const std::li
 	const Sprite *fast = SpriteSet::Get("ui/fast forward");
 	bool separated = false;
 
-	for(auto it = list.begin(); it != list.end(); ++it)
+	for(auto it = missionList.begin(); it != missionList.end(); ++it)
 	{
 		if(!it->IsVisible())
 			continue;
@@ -884,7 +887,7 @@ Point MissionPanel::DrawList(const list<Mission> &list, Point pos, const std::li
 			SpriteShader::Draw(fast, pos + Point(-4., 8.));
 
 		const Color *color = nullptr;
-		bool canAccept = (&list == &available ? it->CanAccept(player) : IsSatisfied(*it));
+		bool canAccept = (&missionList == &available ? it->CanAccept(player) : IsSatisfied(*it));
 		if(!canAccept)
 		{
 			if(it->Unavailable().IsLoaded())
@@ -907,7 +910,7 @@ Point MissionPanel::DrawList(const list<Mission> &list, Point pos, const std::li
 				color = &unselected;
 		}
 
-		font.Draw({it->Name(), {SIDE_WIDTH - 11, Truncate::BACK}}, pos, *color);
+		font.Draw({it->DisplayName(), {SIDE_WIDTH - 11, Truncate::BACK}}, pos, *color);
 	}
 
 	return pos;
@@ -1042,7 +1045,7 @@ void MissionPanel::Accept(bool force)
 		else
 			out << "You must sell " << Format::CargoString(cargoToSell, "ordinary commodities")
 				<< " to make room for this mission. Continue?";
-		GetUI()->Push(new Dialog(this, &MissionPanel::MakeSpaceAndAccept, out.str()));
+		GetUI().Push(new DialogPanel(this, &MissionPanel::MakeSpaceAndAccept, out.str()));
 		return;
 	}
 
@@ -1217,6 +1220,13 @@ void MissionPanel::CycleInvolvedSystems(const Mission &mission)
 			return;
 		}
 
+	for(const System *mark : mission.TrackedSystems())
+		if(++index == cycleInvolvedIndex)
+		{
+			CenterOnSystem(mark);
+			return;
+		}
+
 	cycleInvolvedIndex = 0;
 	CenterOnSystem(mission.Destination()->GetSystem());
 }
