@@ -26,6 +26,10 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "Logger.h"
 #include "Screen.h"
 
+#ifdef _WIN32
+#include "windows/WinVersion.h"
+#endif
+
 #include <algorithm>
 #include <cstddef>
 #include <map>
@@ -35,6 +39,7 @@ using namespace std;
 namespace {
 	map<string, bool> settings;
 	int scrollSpeed = 60;
+	int tooltipActivation = 60;
 
 	// Strings for ammo expenditure:
 	const string EXPEND_AMMO = "Escorts expend ammo";
@@ -84,7 +89,7 @@ namespace {
 
 		const bool IsActive() const { return state != Preferences::OverlayState::DISABLED; }
 
-		const std::string &ToString() const
+		const string &ToString() const
 		{
 			return OVERLAY_SETTINGS[max<int>(0, min<int>(OVERLAY_SETTINGS.size() - 1, static_cast<int>(state)))];
 		}
@@ -170,7 +175,20 @@ namespace {
 	const vector<string> FLAGSHIP_SPACE_PRIORITY_SETTINGS = {"none", "passengers", "cargo", "both"};
 	int flagshipSpacePriorityIndex = 1;
 
+	const vector<string> LARGE_GRAPHICS_REDUCTION_SETTINGS = {"off", "largest only", "all"};
+	int largeGraphicsReductionIndex = 0;
+
+	const string BLOCK_SCREEN_SAVER = "Block screen saver";
+
 	int previousSaveCount = 3;
+
+#ifdef _WIN32
+	const vector<string> TITLE_BAR_THEME_SETTINGS = {"system default", "light", "dark"};
+	int titleBarThemeIndex = 0;
+
+	const vector<string> WINDOW_ROUNDING_SETTINGS = {"system default", "off", "large", "small"};
+	int windowRoundingIndex = 0;
+#endif
 }
 
 
@@ -207,13 +225,15 @@ void Preferences::Load()
 		const string &key = node.Token(0);
 		bool hasValue = node.Size() >= 2;
 		if(key == "window size" && node.Size() >= 3)
-			Screen::SetRaw(node.Value(1), node.Value(2));
+			Screen::SetRaw(node.Value(1), node.Value(2), true);
 		else if(key == "zoom" && hasValue)
-			Screen::SetZoom(node.Value(1));
+			Screen::SetZoom(node.Value(1), true);
 		else if(VOLUME_SETTINGS.contains(key) && hasValue)
 			Audio::SetVolume(node.Value(1) * VOLUME_SCALE, VOLUME_SETTINGS.at(key));
 		else if(key == "scroll speed" && hasValue)
 			scrollSpeed = node.Value(1);
+		else if(key == "Tooltip activation time" && hasValue)
+			tooltipActivation = node.Value(1);
 		else if(key == "boarding target")
 			boardingIndex = max<int>(0, min<int>(node.Value(1), BOARDING_SETTINGS.size() - 1));
 		else if(key == "Flotsam collection")
@@ -254,12 +274,20 @@ void Preferences::Load()
 			minimapDisplayIndex = max<int>(0, min<int>(node.Value(1), MINIMAP_DISPLAY_SETTING.size() - 1));
 		else if(key == "Prioritize flagship use")
 			flagshipSpacePriorityIndex = clamp<int>(node.Value(1), 0, FLAGSHIP_SPACE_PRIORITY_SETTINGS.size() - 1);
+		else if(key == "Reduce large graphics")
+			largeGraphicsReductionIndex = clamp<int>(node.Value(1), 0, LARGE_GRAPHICS_REDUCTION_SETTINGS.size() - 1);
 		else if(key == "previous saves" && hasValue)
 			previousSaveCount = max<int>(3, node.Value(1));
 		else if(key == "alt-mouse turning")
 			settings["Control ship with mouse"] = (!hasValue || node.Value(1));
 		else if(key == "notification settings")
 			notifOptionsIndex = max<int>(0, min<int>(node.Value(1), NOTIF_OPTIONS.size() - 1));
+#ifdef _WIN32
+		else if(key == "Title bar theme")
+			titleBarThemeIndex = clamp<int>(node.Value(1), 0, TITLE_BAR_THEME_SETTINGS.size() - 1);
+		else if(key == "Window rounding")
+			windowRoundingIndex = clamp<int>(node.Value(1), 0, WINDOW_ROUNDING_SETTINGS.size() - 1);
+#endif
 		else
 			settings[key] = (node.Size() == 1 || node.Value(1));
 	}
@@ -306,6 +334,7 @@ void Preferences::Save()
 	out.Write("window size", Screen::RawWidth(), Screen::RawHeight());
 	out.Write("zoom", Screen::UserZoom());
 	out.Write("scroll speed", scrollSpeed);
+	out.Write("Tooltip activation time", tooltipActivation);
 	out.Write("boarding target", boardingIndex);
 	out.Write("Flotsam collection", flotsamIndex);
 	out.Write("view zoom", zoomIndex);
@@ -326,7 +355,14 @@ void Preferences::Save()
 	out.Write("alert indicator", alertIndicatorIndex);
 	out.Write("Show mini-map", minimapDisplayIndex);
 	out.Write("Prioritize flagship use", flagshipSpacePriorityIndex);
+	out.Write("Reduce large graphics", largeGraphicsReductionIndex);
 	out.Write("previous saves", previousSaveCount);
+#ifdef _WIN32
+	if(WinVersion::SupportsDarkTheme())
+		out.Write("Title bar theme", titleBarThemeIndex);
+	if(WinVersion::SupportsWindowRounding())
+		out.Write("Window rounding", windowRoundingIndex);
+#endif
 
 	for(const auto &it : settings)
 		out.Write(it.first, it.second);
@@ -429,6 +465,20 @@ void Preferences::SetScrollSpeed(int speed)
 
 
 
+int Preferences::TooltipActivation()
+{
+	return tooltipActivation;
+}
+
+
+
+void Preferences::SetTooltipActivation(int steps)
+{
+	tooltipActivation = steps;
+}
+
+
+
 // View zoom.
 double Preferences::ViewZoom()
 {
@@ -572,7 +622,7 @@ bool Preferences::ToggleVSync()
 		if(!GameWindow::SetVSync(static_cast<VSync>(targetIndex)))
 		{
 			// Restore original saved setting.
-			Logger::LogError("Unable to change VSync state");
+			Logger::Log("Unable to change VSync state.", Logger::Level::WARNING);
 			GameWindow::SetVSync(static_cast<VSync>(vsyncIndex));
 			return false;
 		}
@@ -781,7 +831,7 @@ Preferences::AlertIndicator Preferences::GetAlertIndicator()
 
 
 
-const std::string &Preferences::AlertSetting()
+const string &Preferences::AlertSetting()
 {
 	return ALERT_INDICATOR_SETTING[alertIndicatorIndex];
 }
@@ -836,7 +886,7 @@ Preferences::MinimapDisplay Preferences::GetMinimapDisplay()
 
 
 
-const std::string &Preferences::MinimapSetting()
+const string &Preferences::MinimapSetting()
 {
 	return MINIMAP_DISPLAY_SETTING[minimapDisplayIndex];
 }
@@ -862,3 +912,81 @@ const string &Preferences::FlagshipSpacePrioritySetting()
 {
 	return FLAGSHIP_SPACE_PRIORITY_SETTINGS[flagshipSpacePriorityIndex];
 }
+
+
+
+void Preferences::ToggleLargeGraphicsReduction()
+{
+	if(++largeGraphicsReductionIndex >= static_cast<int>(LARGE_GRAPHICS_REDUCTION_SETTINGS.size()))
+		largeGraphicsReductionIndex = 0;
+}
+
+
+
+Preferences::LargeGraphicsReduction Preferences::GetLargeGraphicsReduction()
+{
+	return static_cast<LargeGraphicsReduction>(largeGraphicsReductionIndex);
+}
+
+
+
+const string &Preferences::LargeGraphicsReductionSetting()
+{
+	return LARGE_GRAPHICS_REDUCTION_SETTINGS[largeGraphicsReductionIndex];
+}
+
+
+
+void Preferences::ToggleBlockScreenSaver()
+{
+	GameWindow::ToggleBlockScreenSaver();
+	Set(BLOCK_SCREEN_SAVER, !Has(BLOCK_SCREEN_SAVER));
+}
+
+
+
+#ifdef _WIN32
+void Preferences::ToggleTitleBarTheme()
+{
+	if(++titleBarThemeIndex >= static_cast<int>(TITLE_BAR_THEME_SETTINGS.size()))
+		titleBarThemeIndex = 0;
+	GameWindow::UpdateTitleBarTheme();
+}
+
+
+
+Preferences::TitleBarTheme Preferences::GetTitleBarTheme()
+{
+	return static_cast<TitleBarTheme>(titleBarThemeIndex);
+}
+
+
+
+const string &Preferences::TitleBarThemeSetting()
+{
+	return TITLE_BAR_THEME_SETTINGS[titleBarThemeIndex];
+}
+
+
+
+void Preferences::ToggleWindowRounding()
+{
+	if(++windowRoundingIndex >= static_cast<int>(WINDOW_ROUNDING_SETTINGS.size()))
+		windowRoundingIndex = 0;
+	GameWindow::UpdateWindowRounding();
+}
+
+
+
+Preferences::WindowRounding Preferences::GetWindowRounding()
+{
+	return static_cast<WindowRounding>(windowRoundingIndex);
+}
+
+
+
+const string &Preferences::WindowRoundingSetting()
+{
+	return WINDOW_ROUNDING_SETTINGS[windowRoundingIndex];
+}
+#endif
