@@ -19,7 +19,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "CategoryList.h"
 #include "CategoryType.h"
 #include "Color.h"
-#include "Dialog.h"
+#include "DialogPanel.h"
 #include "text/DisplayText.h"
 #include "shader/FillShader.h"
 #include "text/Font.h"
@@ -45,7 +45,6 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "shader/SpriteShader.h"
 #include "text/Truncate.h"
 #include "UI.h"
-#include "text/WrappedText.h"
 
 #include "opengl.h"
 #include <SDL2/SDL.h>
@@ -87,6 +86,8 @@ ShopPanel::ShopPanel(PlayerInfo &player, bool isOutfitter)
 		GameData::Colors().Get("tooltip background"), GameData::Colors().Get("medium")),
 	creditsTooltip(250, Alignment::LEFT, Tooltip::Direction::UP_LEFT, Tooltip::Corner::TOP_RIGHT,
 		GameData::Colors().Get("tooltip background"), GameData::Colors().Get("medium")),
+	buttonsTooltip(250, Alignment::LEFT, Tooltip::Direction::DOWN_LEFT, Tooltip::Corner::TOP_LEFT,
+		GameData::Colors().Get("tooltip background"), GameData::Colors().Get("medium")),
 	hover(*GameData::Colors().Get("hover")),
 	active(*GameData::Colors().Get("active")),
 	inactive(*GameData::Colors().Get("inactive")),
@@ -102,7 +103,7 @@ ShopPanel::ShopPanel(PlayerInfo &player, bool isOutfitter)
 
 void ShopPanel::Step()
 {
-	if(!checkedHelp && GetUI()->IsTop(this) && player.Ships().size() > 1)
+	if(!checkedHelp && GetUI().IsTop(this) && player.Ships().size() > 1)
 	{
 		if(DoHelp("multiple ships"))
 		{
@@ -141,6 +142,13 @@ void ShopPanel::Draw()
 	DrawButtons();
 	DrawKey();
 
+	// Draw the Find button. Note: buttonZones are cleared in DrawButtons.
+	const Point findCenter = Screen::BottomRight() - Point(580, 20);
+	const Sprite *findIcon = SpriteSet::Get(hoverButton == 'f' ? "ui/find selected" : "ui/find unselected");
+	SpriteShader::Draw(findIcon, findCenter);
+	buttonZones.emplace_back(Rectangle(findCenter, {findIcon->Width(), findIcon->Height()}), 'f');
+	static const string FIND = "_Find";
+
 	shipInfo.DrawTooltips();
 	outfitInfo.DrawTooltips();
 
@@ -189,6 +197,7 @@ void ShopPanel::UpdateTooltipActivation()
 {
 	shipsTooltip.UpdateActivationCount();
 	creditsTooltip.UpdateActivationCount();
+	buttonsTooltip.UpdateActivationCount();
 }
 
 
@@ -222,56 +231,47 @@ void ShopPanel::DrawShip(const Ship &ship, const Point &center, bool isSelected)
 
 
 
-void ShopPanel::CheckForMissions(Mission::Location location)
+void ShopPanel::CheckForMissions(Mission::Location location) const
 {
-	if(!GetUI()->IsTop(this))
+	if(!GetUI().IsTop(this))
 		return;
 
 	Mission *mission = player.MissionToOffer(location);
 	if(mission)
-		mission->Do(Mission::OFFER, player, GetUI());
+		mission->Do(Mission::OFFER, player, &GetUI());
 	else
 		player.HandleBlockedMissions(location, GetUI());
 }
 
 
 
-void ShopPanel::FailSell(bool toStorage) const
-{
-}
-
-
-
-bool ShopPanel::CanSellMultiple() const
-{
-	return true;
-}
-
-
-
-// Helper function for UI buttons to determine if the selected item is
-// already owned. Affects if "Install" is shown for already owned items
-// or if "Buy" is shown for items not yet owned.
-//
-// If we are buying into cargo, then items in cargo don't count as already
-// owned, but they count as "already installed" in cargo.
-bool ShopPanel::IsAlreadyOwned() const
-{
-	return (playerShip && selectedOutfit && player.Cargo().Get(selectedOutfit))
-		|| player.Storage().Get(selectedOutfit);
-}
-
-
-
-bool ShopPanel::ShouldHighlight(const Ship *ship)
-{
-	return (hoverButton == 's');
-}
-
-
-
-void ShopPanel::DrawKey()
+void ShopPanel::ValidateSelectedShips()
 {
+	// Verify that the player's selection is still valid.
+	// A mission action may have taken a ship away from the player,
+	// therefore invalidating its pointer.
+	set<Ship *> shipPtrs;
+	for(const shared_ptr<Ship> &ship : player.Ships())
+		shipPtrs.insert(ship.get());
+	set<Ship *> stillValid;
+	ranges::set_intersection(shipPtrs, playerShips, inserter(stillValid, stillValid.begin()));
+	playerShips = stillValid;
+	if(playerShip && !playerShips.contains(playerShip))
+	{
+		playerShip = nullptr;
+		if(!playerShips.empty())
+			playerShip = *playerShips.begin();
+		else
+		{
+			for(const shared_ptr<Ship> &ship : player.Ships())
+				if(CanShowInSidebar(*ship, player.GetPlanet()))
+				{
+					playerShip = ship.get();
+					playerShips.insert(playerShip);
+					break;
+				}
+		}
+	}
 }
 
 
@@ -283,26 +283,9 @@ int ShopPanel::VisibilityCheckboxesSize() const
 
 
 
-void ShopPanel::ToggleForSale()
+bool ShopPanel::ShouldHighlight(const Ship *ship)
 {
-	CheckSelection();
-	delayedAutoScroll = true;
-}
-
-
-
-void ShopPanel::ToggleStorage()
-{
-	CheckSelection();
-	delayedAutoScroll = true;
-}
-
-
-
-void ShopPanel::ToggleCargo()
-{
-	CheckSelection();
-	delayedAutoScroll = true;
+	return (hoverButton == 's' || hoverButton == 'r');
 }
 
 
@@ -310,13 +293,12 @@ void ShopPanel::ToggleCargo()
 // Only override the ones you need; the default action is to return false.
 bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress)
 {
-	bool toStorage = planet && planet->HasOutfitter() && (key == 'r' || key == 'u');
 	if(key == 'l' || key == 'd' || key == SDLK_ESCAPE
 			|| (key == 'w' && (mod & (KMOD_CTRL | KMOD_GUI))))
 	{
 		if(!isOutfitter)
 			player.UpdateCargoCapacities();
-		GetUI()->Pop(this);
+		GetUI().Pop(this);
 		UI::PlaySound(UI::UISound::NORMAL);
 	}
 	else if(command.Has(Command::HELP))
@@ -351,42 +333,9 @@ bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 	else if(command.Has(Command::MAP))
 	{
 		if(isOutfitter)
-			GetUI()->Push(new MapOutfitterPanel(player));
+			GetUI().Push(new MapOutfitterPanel(player));
 		else
-			GetUI()->Push(new MapShipyardPanel(player));
-	}
-	else if(key == 'b' || key == 'i' || key == 'c')
-	{
-		const auto result = CanBuy(key == 'i' || key == 'c');
-		if(result)
-		{
-			Buy(key == 'i' || key == 'c');
-			// Ship-based updates to cargo are handled when leaving.
-			// Ship-based selection changes are asynchronous, and handled by ShipyardPanel.
-			if(isOutfitter)
-			{
-				player.UpdateCargoCapacities();
-				CheckSelection();
-			}
-		}
-		else if(result.HasMessage())
-			GetUI()->Push(new Dialog(result.Message()));
-	}
-	else if(key == 's' || toStorage)
-	{
-		if(!CanSell(toStorage))
-			FailSell(toStorage);
-		else
-		{
-			int modifier = CanSellMultiple() ? Modifier() : 1;
-			for(int i = 0; i < modifier && CanSell(toStorage); ++i)
-				Sell(toStorage);
-			if(isOutfitter)
-			{
-				player.UpdateCargoCapacities();
-				CheckSelection();
-			}
-		}
+			GetUI().Push(new MapShipyardPanel(player));
 	}
 	else if(key == SDLK_LEFT)
 	{
@@ -441,12 +390,12 @@ bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 	{
 		int group = key - '0';
 		if(mod & (KMOD_CTRL | KMOD_GUI))
-			player.SetGroup(group, &playerShips);
+			player.SetEscortGroup(group, &playerShips);
 		else if(mod & KMOD_SHIFT)
 		{
 			// If every single ship in this group is already selected, shift
 			// plus the group number means to deselect all those ships.
-			set<Ship *> added = player.GetGroup(group);
+			set<Ship *> added = player.GetEscortGroup(group);
 			bool allWereSelected = true;
 			for(Ship *ship : added)
 				allWereSelected &= playerShips.erase(ship);
@@ -466,7 +415,7 @@ bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 		{
 			// Change the selection to the desired ships, if they are landed here.
 			playerShips.clear();
-			set<Ship *> wanted = player.GetGroup(group);
+			set<Ship *> wanted = player.GetEscortGroup(group);
 
 			const Planet *here = player.GetPlanet();
 			for(Ship *ship : wanted)
@@ -480,9 +429,21 @@ bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 	else if(key == SDLK_TAB)
 		activePane = (activePane == ShopPane::Main ? ShopPane::Sidebar : ShopPane::Main);
 	else if(key == 'f')
-		GetUI()->Push(new Dialog(this, &ShopPanel::DoFind, "Search for:"));
+		GetUI().Push(new DialogPanel(this, &ShopPanel::DoFind, "Search for:"));
 	else
-		return false;
+	{
+		TransactionResult result = HandleShortcuts(key);
+		if(result.HasMessage())
+			GetUI().Push(new DialogPanel(result.Message()));
+		else if(isOutfitter)
+		{
+			// Ship-based updates to cargo are handled when leaving.
+			// Ship-based selection changes are asynchronous, and handled by ShipyardPanel.
+			player.UpdateCargoCapacities();
+		}
+	}
+
+	CheckSelection();
 
 	return true;
 }
@@ -507,6 +468,7 @@ bool ShopPanel::Click(int x, int y, MouseButton button, int clicks)
 		return false;
 
 	dragShip = nullptr;
+
 	// Handle clicks on the buttons.
 	char zoneButton = CheckButton(x, y);
 	if(zoneButton)
@@ -576,7 +538,6 @@ bool ShopPanel::Click(int x, int y, MouseButton button, int clicks)
 			return true;
 		}
 
-
 	return true;
 }
 
@@ -680,7 +641,7 @@ void ShopPanel::DoFind(const string &text)
 	int index = FindItem(text);
 	if(index >= 0 && index < static_cast<int>(zones.size()))
 	{
-		auto best = std::next(zones.begin(), index);
+		auto best = next(zones.begin(), index);
 		if(best->GetShip())
 			selectedShip = best->GetShip();
 		else
@@ -696,10 +657,12 @@ void ShopPanel::DoFind(const string &text)
 
 int64_t ShopPanel::LicenseCost(const Outfit *outfit, bool onlyOwned) const
 {
+	// onlyOwned represents that `outfit` is being transferred from Cargo or Storage.
+
 	// If the player is attempting to install an outfit from cargo, storage, or that they just
 	// sold to the shop, then ignore its license requirement, if any. (Otherwise there
 	// would be no way to use or transfer license-restricted outfits between ships.)
-	bool owned = (player.Cargo().Get(outfit) && playerShip) || player.Storage().Get(outfit);
+	bool owned = player.Cargo().Get(outfit) || player.Storage().Get(outfit);
 	if((owned && onlyOwned) || player.Stock(outfit) > 0)
 		return 0;
 
@@ -830,7 +793,7 @@ void ShopPanel::DrawShipsSidebar()
 
 		shipZones.emplace_back(point, Point(ICON_TILE, ICON_TILE), ship.get());
 
-		if(mouse.Y() < Screen::Bottom() - BUTTON_HEIGHT && shipZones.back().Contains(mouse))
+		if(mouse.Y() < Screen::Bottom() - ButtonPanelHeight() && shipZones.back().Contains(mouse))
 		{
 			shipName = ship->GivenName() + (ship->IsParked() ? "\n" + GameData::Tooltip("parked") : "");
 			shipsTooltip.SetZone(shipZones.back());
@@ -871,16 +834,7 @@ void ShopPanel::DrawShipsSidebar()
 		const int detailHeight = DrawPlayerShipInfo(point + offset);
 		point.Y() += detailHeight + SHIP_SIZE / 2;
 	}
-	else if(isOutfitter && player.Cargo().Size())
-	{
-		point.X() = Screen::Right() - SIDEBAR_WIDTH + 10;
-		font.Draw("cargo space:", point, medium);
-
-		string space = Format::Number(player.Cargo().Free()) + " / " + Format::Number(player.Cargo().Size());
-		font.Draw({space, {SIDEBAR_WIDTH - 20, Alignment::RIGHT}}, point, bright);
-		point.Y() += 20.;
-	}
-	sidebarScroll.SetDisplaySize(Screen::Height() - BUTTON_HEIGHT);
+	sidebarScroll.SetDisplaySize(Screen::Height() - ButtonPanelHeight());
 	sidebarScroll.SetMaxValue(max(0., point.Y() + sidebarScroll.AnimatedValue() - Screen::Bottom() + Screen::Height()));
 
 	if(sidebarScroll.Scrollable())
@@ -931,83 +885,6 @@ void ShopPanel::DrawDetailsSidebar()
 
 
 
-void ShopPanel::DrawButtons()
-{
-	// The last 70 pixels on the end of the side panel are for the buttons:
-	Point buttonSize(SIDEBAR_WIDTH, BUTTON_HEIGHT);
-	FillShader::Fill(Screen::BottomRight() - .5 * buttonSize, buttonSize,
-		*GameData::Colors().Get("shop side panel background"));
-	FillShader::Fill(
-		Point(Screen::Right() - SIDEBAR_WIDTH / 2, Screen::Bottom() - BUTTON_HEIGHT),
-		Point(SIDEBAR_WIDTH, 1), *GameData::Colors().Get("shop side panel footer"));
-
-	const Font &font = FontSet::Get(14);
-	const Color &bright = *GameData::Colors().Get("bright");
-	const Color &dim = *GameData::Colors().Get("medium");
-
-	const Point creditsPoint(
-		Screen::Right() - SIDEBAR_WIDTH + 10,
-		Screen::Bottom() - 65);
-	font.Draw("You have:", creditsPoint, dim);
-
-	const auto credits = Format::CreditString(player.Accounts().Credits());
-	font.Draw({credits, {SIDEBAR_WIDTH - 20, Alignment::RIGHT}}, creditsPoint, bright);
-
-	// Clear the buttonZones, they will be populated again as buttons are drawn.
-	buttonZones.clear();
-
-	const Point buyCenter = Screen::BottomRight() - Point(210, 25);
-	bool isOwned = IsAlreadyOwned();
-
-	if(isOwned)
-		ShopPanel::DrawButton(playerShip ? "_Install" : "_Cargo", Rectangle(buyCenter, Point(60, 30)),
-			static_cast<bool>(CanBuy(isOwned)), hoverButton == 'i', 'i');
-	else
-		ShopPanel::DrawButton("_Buy", Rectangle(buyCenter, Point(60, 30)),
-			static_cast<bool>(CanBuy(isOwned)), hoverButton == 'b', 'b');
-
-	const Point sellCenter = Screen::BottomRight() - Point(130, 25);
-	ShopPanel::DrawButton("_Sell", Rectangle(sellCenter, Point(60, 30)),
-		static_cast<bool>(CanSell()), hoverButton == 's', 's');
-
-	const Point leaveCenter = Screen::BottomRight() - Point(45, 25);
-	ShopPanel::DrawButton("_Leave", Rectangle(leaveCenter, Point(70, 30)),
-		true, hoverButton == 'l', 'l');
-
-	const Point findCenter = Screen::BottomRight() - Point(580, 20);
-	const Sprite *findIcon =
-		hoverButton == 'f' ? SpriteSet::Get("ui/find selected") : SpriteSet::Get("ui/find unselected");
-	SpriteShader::Draw(findIcon, findCenter);
-	buttonZones.emplace_back(Rectangle(findCenter, {findIcon->Width(), findIcon->Height()}), 'f');
-	static const string FIND = "_Find";
-
-	int modifier = Modifier();
-	if(modifier > 1)
-	{
-		string mod = "x " + to_string(modifier);
-		int modWidth = font.Width(mod);
-		font.Draw(mod, buyCenter + Point(-.5 * modWidth, 10.), dim);
-		if(CanSellMultiple())
-			font.Draw(mod, sellCenter + Point(-.5 * modWidth, 10.), dim);
-	}
-
-	// Draw the tooltip for your full number of credits.
-	const Rectangle creditsBox = Rectangle::FromCorner(creditsPoint, Point(SIDEBAR_WIDTH - 20, 15));
-	if(creditsBox.Contains(hoverPoint))
-		creditsTooltip.IncrementCount();
-	else
-		creditsTooltip.DecrementCount();
-
-	if(creditsTooltip.ShouldDraw())
-	{
-		creditsTooltip.SetZone(creditsBox);
-		creditsTooltip.SetText(Format::Number(player.Accounts().Credits()) + " credits", true);
-		creditsTooltip.Draw();
-	}
-}
-
-
-
 void ShopPanel::DrawMain()
 {
 	const Font &bigFont = FontSet::Get(18);
@@ -1099,10 +976,12 @@ void ShopPanel::DrawMain()
 	nextY -= 40 + TILE_SIZE;
 
 	// What amount would mainScroll have to equal to make nextY equal the
-	// bottom of the screen? (Also leave space for the "key" at the bottom.)
+	// bottom of the screen? (Also leave space for the "key" at the bottom,
+	// and a small (10px) amount of space between the last item and the bottom
+	// of the screen.)
 	mainScroll.SetDisplaySize(Screen::Height());
 	mainScroll.SetMaxValue(max(0., nextY + mainScroll.AnimatedValue() - Screen::Height() / 2 - TILE_SIZE / 2 +
-		VisibilityCheckboxesSize() + 40.) + Screen::Height());
+		VisibilityCheckboxesSize() + 10.) + Screen::Height());
 
 	if(mainScroll.Scrollable())
 	{
@@ -1590,9 +1469,9 @@ char ShopPanel::CheckButton(int x, int y)
 		if(zone.Contains(clickPoint))
 			return zone.Value();
 
-	if(x < Screen::Right() - SIDEBAR_WIDTH || y < Screen::Bottom() - BUTTON_HEIGHT)
+	if(x < Screen::Right() - SIDEBAR_WIDTH || y < Screen::Bottom() - ButtonPanelHeight())
 		return '\0';
 
-	// Returning space here ensures that hover text for the ship info panel is supressed.
+	// Returning space here ensures that hover text for the ship info panel is suppressed.
 	return ' ';
 }
